@@ -29,8 +29,11 @@ class Syncer:
             self.src_hashes = HashTracker.from_file(src)
             if self.hash_file.exists():
                 self.dst_hashes = HashTracker.from_serialized(dst, self.hash_file.read_text())
+                logging.log(f"Reading hashes from {self.hash_file}")
             else:
                 self.dst_hashes = HashTracker(dst)
+                self.hash_file.write_text(self.dst_hashes.serialise())
+                logging.log(f"Creating empty hashes file at {self.hash_file}")
 
     def _check_wrong_file_type(self, dst: Path, should_be_dir: bool) -> bool:
         if dst.exists() and (dst.is_dir() != should_be_dir):
@@ -73,6 +76,7 @@ class Syncer:
     def update_hashes(self, path: Path):
         if self.use_hashes:
             self.dst_hashes.update_hash(path, self.src_hashes)
+            print(f"Updating hash for {path}")
 
     def check_hashes(self, path: Path) -> bool:
         return (self.src_hashes.get_hash(path)
@@ -97,14 +101,15 @@ class Syncer:
 
         return self.force
 
-    def autosave_hashes(self):
+    def autosave_hashes(self, force=False):
         if not self.use_hashes:
             return
-        if time.time() - self.last_autosave_time < 2:
+        if force or time.time() - self.last_autosave_time < 2:
             return
 
         self.last_autosave_time = time.time()
 
+        terminal_formatting.print_temp("Saving hashes to destination")
         self.hash_file.write_text(self.dst_hashes.serialise())
 
     def visit_file(self, path: Path):
@@ -170,6 +175,9 @@ class Syncer:
 
         file.unlink(missing_ok=True)
 
+    def finish(self):
+        self.autosave_hashes(force=True)
+
 
 def sync(src, dest, verbose=True, do_delete=False, check_metadata=True,
          advanced_output_features=True, use_hash: bool = False):
@@ -184,5 +192,6 @@ def sync(src, dest, verbose=True, do_delete=False, check_metadata=True,
     :param use_hash: Whether to store the hashes of the copied files in a small file in the destination
     """
 
-    (Syncer(Path(src), Path(dest), verbose, do_delete, check_metadata, advanced_output_features, use_hash)
-     .visit(Path(".")))
+    syncer = Syncer(Path(src), Path(dest), verbose, do_delete, check_metadata, advanced_output_features, use_hash)
+    syncer.visit(Path("."))
+    syncer.finish()
